;Copyright (C) 2004-2009 John T. Haller of PortableApps.com
;Copyright (C) 2007-2009 Ryan McCue of PortableApps.com
;Copyright (C) 2007-2009 MStinaff

;Website: http://portableapps.com/DevelopmentTest

;This software is OSI Certified Open Source Software.
;OSI Certified is a certification mark of the Open Source Initiative.

;This program is free software; you can redistribute it and/or
;modify it under the terms of the GNU General Public License
;as published by the Free Software Foundation; either version 2
;of the License, or (at your option) any later version.

;This program is distributed in the hope that it will be useful,
;but WITHOUT ANY WARRANTY; without even the implied warranty of
;MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;GNU General Public License for more details.

;You should have received a copy of the GNU General Public License
;along with this program; if not, write to the Free Software
;Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

!define PORTABLEAPPNAME "WinPT Portable"
!define APPNAME "WinPT"
!define NAME "WinPTPortable"
!define VER "1.4.2.0"
!define WEBSITE "PortableApps.com/DevelopmentTest"
!define DEFAULTEXE "WinPT.exe"
!define DEFAULTAPPDIR "WinPT"
!define DEFAULTSETTINGSDIR "settings"
!define LAUNCHERLANGUAGE "English"
!define GNUPGPORTABLEPATH "\PortableApps\CommonFiles\GnuPG\GnuPGPortable.exe"

;=== Program Details
Name "${PORTABLEAPPNAME}"
OutFile "..\..\${NAME}.exe"
Caption "${PORTABLEAPPNAME} | PortableApps.com"
VIProductVersion "${VER}"
VIAddVersionKey ProductName "${PORTABLEAPPNAME}"
VIAddVersionKey Comments "Allows ${APPNAME} to be run from a removable drive.  For additional details, visit ${WEBSITE}"
VIAddVersionKey CompanyName "PortableApps.com"
VIAddVersionKey LegalCopyright "PortableApps.com & Contributors"
VIAddVersionKey FileDescription "${PORTABLEAPPNAME}"
VIAddVersionKey FileVersion "${VER}"
VIAddVersionKey ProductVersion "${VER}"
VIAddVersionKey InternalName "${PORTABLEAPPNAME}"
VIAddVersionKey LegalTrademarks "PortableApps.com is a Trademark of Rare Ideas, LLC."
VIAddVersionKey OriginalFilename "${NAME}.exe"
;VIAddVersionKey PrivateBuild ""
;VIAddVersionKey SpecialBuild ""

;=== Runtime Switches
CRCCheck On
WindowIcon Off
SilentInstall Silent
AutoCloseWindow True
RequestExecutionLevel user

; Best Compression
SetCompress Auto
SetCompressor /SOLID lzma
SetCompressorDictSize 32
SetDatablockOptimize On

;=== Include
;(Standard NSIS)
!include Registry.nsh
!include FileFunc.nsh
!insertmacro GetRoot
!insertmacro GetParameters

;(NSIS Plugins)
!include TextReplace.nsh

;(Custom)
!include ReadINIStrWithDefault.nsh
!include GnuPGPortable.nsh

;=== Program Icon
Icon "..\..\App\AppInfo\appicon.ico"

;=== Languages
LoadLanguageFile "${NSISDIR}\Contrib\Language files\${LAUNCHERLANGUAGE}.nlf"
!include PortableApps.comLauncherLANG_${LAUNCHERLANGUAGE}.nsh

Var PROGRAMDIRECTORY
Var SETTINGSDIRECTORY
Var ADDITIONALPARAMETERS
Var EXECSTRING
Var DISABLESPLASHSCREEN
Var SECONDARYLAUNCH
Var FAILEDTORESTOREKEY
Var MISSINGFILEORPATH
Var DISPLAYPATHS
Var GPGPATH
Var GPGHOME

Section "Main"

		;=== Check if already running
		System::Call 'kernel32::CreateMutexA(i 0, i 0, t "${NAME}") i .r1 ?e'
		Pop $0
		StrCmp $0 0 CheckINI
			StrCpy $SECONDARYLAUNCH "true"

	CheckINI:
		;=== Read WinPTPortable.ini if it exists
		${ReadINIStrWithDefault} $ADDITIONALPARAMETERS "$EXEDIR\${NAME}.ini" "${NAME}" "AdditionalParameters" ""
		${ReadINIStrWithDefault} $DISABLESPLASHSCREEN "$EXEDIR\${NAME}.ini" "${NAME}" "DisableSplashScreen" "false"
		${ReadINIStrWithDefault} $DISPLAYPATHS "$EXEDIR\${NAME}.ini" "${NAME}" "DisplayPaths" "true"
		StrCpy $PROGRAMDIRECTORY "$EXEDIR\App\${DEFAULTAPPDIR}"
		StrCpy $SETTINGSDIRECTORY "$EXEDIR\Data\settings"
		StrCpy $EXECSTRING "$PROGRAMDIRECTORY\${DEFAULTEXE}"

		IfFileExists "$EXECSTRING" FoundProgramEXE NoProgramEXE

	NoProgramEXE:
		;=== Program executable not where expected
		StrCpy $MISSINGFILEORPATH ${DEFAULTEXE}
		MessageBox MB_OK|MB_ICONEXCLAMATION `$(LauncherFileNotFound)`
		Abort

	FoundProgramEXE:
		;=== Check if running
		StrCmp $SECONDARYLAUNCH "true" GetPassedParameters
		FindProcDLL::FindProc ${DEFAULTEXE}
		StrCmp $R0 "1" WarnAnotherInstance DisplaySplash

	WarnAnotherInstance:
		MessageBox MB_OK|MB_ICONINFORMATION `$(LauncherAlreadyRunning)`
		Abort

	DisplaySplash:
		StrCmp $DISABLESPLASHSCREEN "true" GetPassedParameters
			;=== Show the splash screen while processing registry entries
			InitPluginsDir
			File /oname=$PLUGINSDIR\splash.jpg "${NAME}.jpg"
			newadvsplash::show /NOUNLOAD 1200 0 0 -1 /L $PLUGINSDIR\splash.jpg

	GetPassedParameters:
		;=== Get any passed parameters
		${GetParameters} $0
		StrCmp "'$0'" "''" AdditionalParameters
		StrCpy $EXECSTRING `$EXECSTRING $0`

	AdditionalParameters:
		StrCmp $ADDITIONALPARAMETERS "" SettingsDirectory

		;=== Additional Parameters
		StrCpy $EXECSTRING `$EXECSTRING $ADDITIONALPARAMETERS`

	SettingsDirectory:
		;=== Set the settings directory if we have a path
		IfFileExists "$SETTINGSDIRECTORY\*.*" RegistryBackup
			CreateDirectory $SETTINGSDIRECTORY

	RegistryBackup:
		StrCmp $SECONDARYLAUNCH "true" LaunchAndExit
		;=== Backup the registry
		${registry::KeyExists} "HKEY_CURRENT_USER\Software\WinPT-BackupByWinPTPortable" $R0
		StrCmp $R0 "0" RestoreTheKey
		${registry::KeyExists} "HKEY_CURRENT_USER\Software\WinPT" $R0
		StrCmp $R0 "-1" RestoreTheKey
		${registry::MoveKey} "HKEY_CURRENT_USER\Software\WinPT" "HKEY_CURRENT_USER\Software\WinPT-BackupByWinPTPortable" $R0
		Sleep 100

	RestoreTheKey:
		IfFileExists "$SETTINGSDIRECTORY\WinPT.reg" "" SetPathKey
			IfFileExists "$WINDIR\system32\reg.exe" "" RestoreTheKey9x
				nsExec::ExecToStack `"$WINDIR\system32\reg.exe" import "$SETTINGSDIRECTORY\WinPT.reg"`
				Pop $R0
				StrCmp $R0 '0' SetPathKey ;successfully restored key
			RestoreTheKey9x:
				${registry::RestoreKey} "$SETTINGSDIRECTORY\WinPT.reg" $R0
				StrCmp $R0 '0' SetPathKey ;successfully restored key
				StrCpy $FAILEDTORESTOREKEY "true"
				
	SetPathKey:
		${registry::Write} "HKEY_CURRENT_USER\Software\WinPT" "Install Directory" $PROGRAMDIRECTORY "REG_SZ" $R0
		StrCmp $R0 "0" GetGPG StartCleanup
		
	GetGPG:
		;=== Get the path to GnuPG
		${GetGnuPGPath} $GPGPATH $GPGHOME
		StrCmp $GPGPATH "-1" "" DisplayGPG
			StrCpy $MISSINGFILEORPATH "GnuPG"
			MessageBox MB_OK|MB_ICONEXCLAMATION `$(LauncherFileNotFound)`
			Goto StartCleanup
		
	DisplayGPG:
		;=== Display the GPGPath for confirmation unless opted out by ini
		StrCmp $DISPLAYPATHS "false" SetupGPG
			MessageBox MB_YESNO|MB_ICONQUESTION|MB_SETFOREGROUND `Continue with following paths?$\r$\nGnuPG Exec: $GPGPATH$\n$\rGnuPG Home: $GPGHOME` IDYES SetupGPG
				Goto StartCleanup
			
	SetupGPG:
		${SetupGnuPGEnv} $GPGPATH $GPGHOME
		${SetupGnuPGReg} $GPGPATH $GPGHOME

	;Launch:
		Exec $EXECSTRING

	CheckRunning:
		Sleep 1000
		FindProcDLL::FindProc ${DEFAULTEXE}
		StrCmp $R0 "1" CheckRunning

		;=== outside general cleanup.  Only call if SetupGnuPGReg has been called
		${ResetGnuPGReg}

	StartCleanup:
		StrCmp $FAILEDTORESTOREKEY "true" SetOriginalKeyBack
		${registry::SaveKey} "HKEY_CURRENT_USER\Software\WinPT" "$SETTINGSDIRECTORY\WinPT.reg" "" $0
		Sleep 100

	SetOriginalKeyBack:
		${registry::DeleteKey} "HKEY_CURRENT_USER\Software\WinPT" $R0
		Sleep 100
		${registry::KeyExists} "HKEY_CURRENT_USER\Software\WinPT-BackupByWinPTPortable" $R0
		StrCmp $R0 "-1" TheEnd
		${registry::MoveKey} "HKEY_CURRENT_USER\Software\WinPT-BackupByWinPTPortable" "HKEY_CURRENT_USER\Software\WinPT" $R0
		Sleep 100
		Goto TheEnd

	LaunchAndExit:
		Exec $EXECSTRING
	
	TheEnd:
		${registry::Unload}
		newadvsplash::stop /WAIT
		
SectionEnd